function BPDNtable(varargin)
%BPDNTABLE  Generates BPDN comparison table.
%
%   BPDNTABLE(FLAGS,OPTIONS), all FLAGS and OPTIONS are optional
%   and have default values. All FLAGS are false unless set. Valid
%   FLAGS are
%
%   'export'     - Output results to file
%   'nogenerate' - Suppress generating data files when missing
%   
%   OPTIONS consist of a keyword followed by a value:
%
%   'mode' - [0] Generated tables as it appears in the paper
%             1  Display all information
%             2  Display additional debug information
%
%   All flags not used by BPDNtable are passed to BPDNgen and
%   BPDNbatch. The most useful amongst those is the 'parallel' flag
%   for BPDNbatch which enables using parfor when installed.
%
%   Examples:
%   % Show full table including missing entries
%   BPDNtable('nogenerate','mode',1);
%
% Paper: Probing the Pareto Frontier for Basis Pursuit Solutions

%   Copyright 2008, Ewout van den Berg and Michael P. Friedlander
%   $Id$

% ---------------------------------------------------------------

% Parse parameters
[opts,varg] = parseOptions(varargin, {'nogenerate','export'},{'mode'});
mode        = getOption(opts,'mode',0);

% Set mode to 0 if export flag is set
if opts.export, mode = 0; end;

% Generate list of required problem information
plist = BPDNgen(varg);
if mode == 0
   excludeSolvers = {'l1magic'};
   excludeProblem = {'blkheavi','blknheavi','srcsep3'};
   excludeRelres  = [1e-2];
   filesMissing   = 0;
   pidx           = [];
   
   for i=1:length(plist)
      p       = plist{i};
      solver  = p{1};
      relres  = p{2};
      problem = p{3};
      filename= p{4};
      if ismember(solver, excludeSolvers), continue; end;
      if ismember(relres, excludeRelres ), continue; end;
      if ismember(problem,excludeProblem), continue; end;
      pidx(end+1) = i;
      
      % Check existence of file
      if ~exist(filename,'file')
        filesMissing = 1;
      end
   end
else
   filesMissing = 0;
   pidx         = 1:length(plist);
end

% Check if all required files are present
if opts.nogenerate && filesMissing
  error(['Required files are missing, please download the data or ' ...
         'run script without the ''nogenerate'' flag']);
end


% Load or generate the data
soln = cell(1,length(plist));
if ~opts.nogenerate 
   soln(pidx) = BPDNbatch(varg{:},'problems',plist(pidx));
else
   for i = pidx
      p        = plist{i};
      filename = p{4};
      if exist(filename,'file')
        s = load(filename,'stats');
        soln{i} = s.stats;
      end      
   end
end

% ---------------------------------------------------------------
%                        PROCESS THE DATA
% ---------------------------------------------------------------

% Open file for output
if opts.export
   filename = 'tables/BPDNtable.tex';
   fid = fopen(filename,'w');
   if fid == -1
     error(sprintf('Unable to open %s for writing',filename));
   end
   
else
   fid = 1;
end


% Generate list of available problems and solvers
problems = {};
solvers  = {};
for i=1:length(soln)
   if isempty(soln{i}), continue; end;
   problems{end+1} = soln{i}.name;
   solvers{end+1}  = soln{i}.solver;
end
problems = unique(problems);
solvers  = unique(solvers);


% Generate the table
for pidx = 1:length(problems)
    p = problems{pidx};

    % Write out a separator
    if (mode == 0) && (pidx > 1)
        fprintf(fid,'\\midrule\n');
    end
     
    % Get all instances for this problem
    idx    = [];
    sigmas = [];
    for i=1:length(soln)
        if ~isempty(soln{i}) && strcmp(soln{i}.name,p)
            idx(end+1) = i;
            sigmas(end+1) = soln{i}.sigma;
        end
    end
    sigmas = unique(sigmas);
    sigmas = sort(sigmas,'descend');
    
    % Populate the table row
    for sgidx = 1:length(sigmas)
        sg = sigmas(sgidx);

        if (sgidx == 1)
            fprintf(fid,'%10s',p);
        else
            fprintf(fid,'%10s','');
        end
        
        
        for sidx = 1:length(solvers)
            s = solvers{sidx};
            
            % Get the solution for this instance (solver,sigma)
            idxsol = NaN;
            for i=idx
                if ~(isempty(soln{i})) && ...
                     (soln{i}.sigma == sg) && strcmp(soln{i}.solver,s)
                    idxsol = i; break;
                end
            end
            
            % Add solver separator
            if sidx > 1
                fprintf(fid,' &');
            end
            
            % Fill the corresponding cells
            if (mode == 2), fprintf(fid,' [%3d] ', idxsol); end
            if ~isnan(idxsol)
                if isempty(soln{idxsol}.err) && ~strcmp(soln{idxsol}.log,'FAILED')
                    fmt = sprintf('& %7.1e & %7.1e & %9d & %9d', ...
                                  soln{idxsol}.rNorm, ...
                                  soln{idxsol}.xNorm, ...
                                  sparsity(soln{idxsol}.x,0.999),...
                                  sum(soln{idxsol}.matvec));
                    %             soln{idxsol}.runTime);
                    fmt = strrep(fmt,'e','\e');
                    fmt = strrep(fmt,'+0','+');
                    fmt = strrep(fmt,'-0','-');
                    fprintf(fid,'%s',fmt);
                else
                    fprintf(fid,'& %7s & %7s & %9s & %9s','{\tt t}','{\tt t}','{\tt t}','{\tt t}'); 
                end
                
            else
                fprintf(fid,'& %7s & %7s & %9s & %9s','x','x','x','x');
            end
        end
        
        if pidx ~= length(problems) || sgidx ~= length(sigmas)
           fprintf(fid,'\\\\\n');
        elseif fid == 1
           fprintf(fid,'\n');
        end
    end    
end

% Close file
if fid ~= 1, fclose(fid); end
