function BPtable(varargin)
%BPTABLE  Generates BP comparison table.
%
%   BPTABLE(FLAGS,OPTIONS), all FLAGS and OPTIONS are optional
%   and have default values. All FLAGS are false unless set. Valid
%   FLAGS are
%
%   'export'     - Output results to file
%   'nogenerate' - Suppress generating data files when missing
%   
%   OPTIONS consist of a keyword followed by a value:
%
%   'mode' - [0] Display all information
%             1  Generate table 1 as it appears in the paper
%             2  Generate table 2 as it appears in the paper
%
%   All flags not used by BPtable are passed to BPgen and
%   BPbatch. The most useful amongst those is the 'parallel' flag
%   for BPbatch which enables using parfor when installed.
%
%   Examples:
%   BPtable('nogenerate','mode',1); % Show table 1 from paper
%
%Paper: Probing the Pareto Frontier for Basis Pursuit Solutions

%   Copyright 2008, Ewout van den Berg and Michael P. Friedlander
%   $Id$

% ---------------------------------------------------------------

% Parse parameters
[opts,varg] = parseOptions(varargin, {'nogenerate','export'},{'mode'});
mode        = getOption(opts,'mode',0);

% Complain when trying to export all information
if opts.export && ~ismember(mode,[1,2])
   error('Please choose mode 1 or 2 when exporting');
end



% Generate list of required problem information
plist = BPgen(varg);
if ismember(mode,[1,2])
   excludeSolvers = {'l1magic','spgl1-2'};
   excludeProblem = {'blkheavi','blknheavi','srcsep3'};
   filesMissing   = 0;
   pidx           = [];
   
   for i=1:length(plist)
      p       = plist{i};
      solver  = p{1};
      problem = p{2};
      filename= p{3};
      if ismember(solver, excludeSolvers), continue; end;
      if ismember(problem,excludeProblem), continue; end;
      pidx(end+1) = i;
      
      % Check existence of file
      if ~exist(filename,'file')
        filesMissing = 1;
      end
   end
else
   filesMissing = 0;
   pidx         = 1:length(plist);
end


% Check if all required files are present
if opts.nogenerate && filesMissing && opts.export
  error(['Required files are missing, please download the data or ' ...
         'run script without the ''nogenerate'' flag']);
end


% Load or generate the data
soln = cell(1,length(plist));
if ~opts.nogenerate 
   soln(pidx) = BPbatch(varg{:},'problems',plist(pidx));
else
   for i = pidx
      p        = plist{i};
      filename = p{3};
      if exist(filename,'file')
        s = load(filename,'stats');
        soln{i} = s.stats;
      end      
   end
end


% ---------------------------------------------------------------
%                        PROCESS THE DATA
% ---------------------------------------------------------------

% Choose display function
if mode == 1
  displayfun = @printTableOne;
elseif mode == 2
  displayfun = @printTableTwo;
else
  displayfun = @printTableAll;  mode = 0;
end

% Open file for output
if opts.export
   if mode == 1
      filename = 'tables/BPtable1.tex';
   else
      filename = 'tables/BPtable2.tex';
   end
   fid = fopen(filename,'w');
   if fid == -1
     error(sprintf('Unable to open %s for writing',filename));
   end
else
   fid = 1;
end


% Generate list of available problems and solvers
problems = {};
solvers  = {};
for i=1:length(soln)
   if isempty(soln{i}), continue; end;
   problems{end+1} = soln{i}.name;
   solvers{end+1}  = soln{i}.solver;
end
problems = unique(problems);
solvers  = unique(solvers);


% Generate the table
for pidx = 1:length(problems)
    p = problems{pidx};
    
    % Get all instances for this problem
    idx = [];
    for i=1:length(soln)
        if isempty(soln{i}), continue; end;
        if strcmp(soln{i}.name,p)
            idx(end+1) = i;
        end
    end

    % Populate the table row
    fprintf(fid,'%10s',p);
        
    for sidx = 1:length(solvers)
        s = solvers{sidx};
            
        % Get the solution for this instance
        idxsol = NaN;
        for i=idx
            if  strcmp(soln{i}.solver,s)
                idxsol = i; break;
            end
        end
            
        % Add solver separator
        fprintf(fid,' &');
            
        % Fill the corresponding cells
        if mode == 0, fprintf(fid,' [%3d] ', idxsol); end
        if ~isnan(idxsol)
            if isempty(soln{idxsol}.err) && ~strcmp(soln{idxsol}.log,'FAILED')
                str = displayfun(soln{idxsol},0);
            else
                str = displayfun([],1);
            end
        else
          str = displayfun([],2);
        end
        fprintf(fid,'%s',str);
    end

    if pidx ~= length(problems)
      fprintf(fid,'\\\\\n');
    elseif fid == 1
      fprintf(fid,'\n');
    end
    
end

if fid ~= 1, fclose(fid); end

end

% ---------------------------------------------------------------
%                    OUTPUT FORMATTING FUNCTIONS
% ---------------------------------------------------------------

% Display table 1
function str = printTableOne(stats,mode)
switch mode
 case 0
  str = sprintf('& %7.1e & %7.1e', ...
              stats.rNorm, stats.xNorm);
  str = strrep(str,'e','\e');
 case 1
  str = sprintf('& %8s & %8s','{\tt t}','{\tt t}'); 
 otherwise
  str = sprintf('& %8s & %8s','x','x'); 
end
end

% Display table 2
function str = printTableTwo(stats,mode)
switch mode
 case 0
  str = sprintf('& %9d & %9d', ...
                sparsity(stats.x,0.999),...
                sum(stats.matvec)...
                );
  str = strrep(str,'e','\e');
 case 1
  str = sprintf('& %7s & %9s','{\tt t}','{\tt t}'); 
 otherwise
  str = sprintf('& %7s & %9s','x','x'); 
end
end

% Display everything
function str = printTableAll(stats,mode)
switch mode
 case 0
  str = sprintf('& %7.1e & %7.1e & %7d & %7d & %7.1e', ...
                stats.rNorm, stats.xNorm, ...
                sum(stats.matvec), ...
                sparsity(stats.x,0.999), ...
                stats.runTime...
                );
  str = strrep(str,'e','\e');
 case 1
  str = sprintf('& %8s & %8s & %7s & %7s & %8s','{\tt t}','{\tt t}','{\tt t}','{\tt t}','{\tt t}'); 
 otherwise
  str = sprintf('& %8s & %8s & %7s & %7s & %8s','x','x','x', 'x','x'); 
end
end
