function DemoComplex(exportflag)
%DEMOCOMPLEX  Plots complex/real BP results for complex problem
%
%   DEMOCOMPLEX(EXPORTFLAG) illustrates the difference between
%   complex basis pursuit and basis pursuit applied to a problem in
%   which the real and imaginary components are split. When the
%   EXPORTFLAG is set the results are output to PDF file.
%
%Paper: Probing the Pareto Frontier for Basis Pursuit Solutions

%   Copyright 2008, Ewout van den Berg and Michael P. Friedlander
%   $Id$

% ---------------------------------------------------------------

% Get parameters
if nargin < 1, exportflag = 0; end;

% Generate problem
randn('state',0);
rand('state',0);

m         = 500;
n         = 2560;
nnz       = 155;
A         = opGaussian(m,n);
x0        = zeros(n,1); 
idx       = randperm(n); idx = idx(1:nnz);
x0(idx)   = randn(nnz,1) + sqrt(-1)*(randn(nnz,1));
b         = A(x0,1);

% Load or compute results
if exist('./data/DemoComplex.mat','file')
   load('./data/DemoComplex.mat');
else
  
   % One-norm of true solution
   tau  = .99*norm(x0,1);
   
   % Use complex vars to find the BP solution
   opts = spgSetParms('optTol',1e-8,'verbosity',1,'iterations',1e4);
   [x1,r1,g1,info1] = spgl1(A,b,tau,[],[],opts);

   % Use variable splitting to approximate the problem.
   M = opTranspose(opSplitComplex(n));
   S = opSplitComplex(m);
   C = opFoG(S,A,M);

   tau  = .99*norm(M(x0,2),1);
   opts = spgSetParms('optTol',1e-8,'verbosity',1,'iterations',1e4);
   [x2,r2,g2,info2] = spgl1(C,[real(b);imag(b)],tau,[],[],opts);
   z2 = M(x2,1);

   save './data/DemoComplex.mat' x1 r1 g1 info1 x2 r2 g2 info2 z2
end


% Plot results
fs = 18; % Font size
ms = 12; % Marker size

figure(1);
h1 = plot(x0(abs(x0)>eps),'bo');   hold on;
h2 = plot(x1(abs(x1)>eps),'r.');   hold off
set(gca,'FontSize',fs); set([h1,h2],'MarkerSize',ms);
legend('exact solution','computed solution');
xlabel('real'); ylabel('imaginary');
box off;

figure(2);
h1 = plot(x0,'bo'); hold on;
h2 = plot(z2,'r.'); hold off
set(gca,'FontSize',fs); set([h1,h2],'MarkerSize',ms);
legend('exact solution','computed solution');
xlabel('real'); ylabel('imaginary');
box off;

% Export plots to file
if exportflag
   figure(1);
   exportPDF('./figures/DemoComplex1.pdf');

   figure(2);
   exportPDF('./figures/DemoComplex2.pdf');
end

% Display some statistics
fprintf('Complex solver:  xNorm = %8e, rNorm = %8e, rGap = %8e\n', ...
        norm(x1,1), norm(r1,2), info1.rGap);
fprintf('RealImag solver: xNorm = %8e, rNorm = %8e, rGap = %8e\n', ...
        norm(z2,1), norm(r2,2), info2.rGap);
