function ParetoExtrap(idx, printflag,legendflags)
%PARETOEXTRAP  Generates Pareto extrapolation plots.
%
%   PARETOEXTRAP(IDX,PRINTFLAG,LEGENDFLAGS) generates the Pareto
%   extrapolation plots for Sparco problem IDX. PRINTFLAG and
%   LEGENDFLAGS are optional. When PRINTFLAG is non-zero the plot
%   are output in PDF format. LEGENDFLAGS is a vector of length
%   five whose entries indicate whether a legend should be output
%   or not.
%
%   Examples:
%   ParetoExtrap(401,0,[0,0,1,0,0]) % Show extraploation for
%                                   % problem 401 as in paper.
%
%Paper: Probing the Pareto Frontier for Basis Pursuit Solutions

%   Copyright 2008, Ewout van den Berg and Michael P. Friedlander
%   $Id$

% ---------------------------------------------------------------

% Check arguments
if nargin < 2, printflag = 0; end;
if nargin < 3, legendflags = [0,0,0,0,0]; end;

% Load results
name    = generateProblem(idx,'getname');
s       = load(sprintf('data/Pareto_%s',name));
s       = s.pstats;
s.rNorm = abs(s.rNorm);

% Generate problem
p = generateProblem(idx);

% Load or generate results
fn = sprintf('data/ParetoExtrap_%s.mat',name);
if exist(fn,'file');
   load(fn);
else
   % Create lambda and sigma values
   lambda       = linspace(norm(p.A(p.b,2),Inf),0,5);
   lambda       = lambda(1:end-1);
   sigma        = linspace(norm(p.b,2),0,5);
   sigma        = sigma(1:end-1);

   lambdaTau    = zeros(size(lambda));
   lambdaRNorm  = ones(size(lambda)) * norm(p.b,2);
   lambdaLambda = lambda;

   sigmaTau     = zeros(size(sigma));
   sigmaRNorm   = ones(size(sigma)) * norm(p.b,2);
   sigmaLambda  = ones(size(sigma)) * lambda(1);


   % Run QPlambda problems
   for i=2:length(lambda)
      stats = QPLSrun(p.A,p.b,lambda(i),[],'l1ls');
      lambdaTau(i)   = stats.xNorm;
      lambdaRNorm(i) = stats.rNorm;
   end

   % Run BPDN problems
   for i=2:length(sigma)
      stats = BPDNrun(p.A,p.b,sigma(i),'spgl1');
      r     = p.b - p.A(stats.x,1);
      sigmaTau(i)   = stats.xNorm;
      sigmaRNorm(i) = stats.rNorm;
      sigmaLambda(i)= norm(p.A(r,2),inf);
   end
   
   % Add smallest BPDN solution
   sigma(end+1)       = abs(s.rNorm(end));
   sigmaTau(end+1)    = s.tau(end);
   sigmaRNorm(end+1)  = abs(s.rNorm(end));
   sigmaLambda(end+1) = s.lambda(end);
   lambda(end+1)      = s.lambda(end);
   lambdaTau(end+1)   = s.tau(end);
   lambdaRNorm(end+1) = abs(s.rNorm(end));
   lambdaLambda(end+1)= s.lambda(end);
   
   save(fn,'lambda','lambdaTau','lambdaRNorm','lambdaLambda', ...
           'sigma', 'sigmaTau', 'sigmaRNorm', 'sigmaLambda');
end

% --------------------------------------------------
% Pareto curve approximation based on several points
% --------------------------------------------------

% Information for tau and lambda based approximation (x,f,g)
Msigma  = [sigmaTau;  sigmaRNorm;  -sigmaLambda./sigmaRNorm]';
Mlambda = [lambdaTau; lambdaRNorm; -lambda./lambdaRNorm    ]';
MPareto = [s.tau,     abs(s.rNorm), s.lambda./abs(s.rNorm) ];

% Pareto curve with samples only
plotcurve(Msigma(1:end-1,:),Mlambda(1:end-1,:),MPareto, [], ...
          'Pareto curve with samples only',...
          'one-norm of solution','two-norm of residual',...
          name,printflag,legendflags(1),1);

% Interpolation using the pchip function
plotcurve(Msigma(1:end-1,:),Mlambda(1:end-1,:),MPareto, @interp_pchip, ...
          'interp_pchip',...
          'one-norm of solution','two-norm of residual',...
          name,printflag,legendflags(2),2);

% Approximation using cubic interpolation, linear extrapolation
plotcurve(Msigma(1:end-1,:),Mlambda(1:end-1,:),MPareto, @interp_qcl, ...
          'interp_qcl',...
          'one-norm of solution','two-norm of residual',...
          name,printflag,legendflags(3),3);

% Approximation using cubic interpolation, linear-quadratic extrapolation
interpfun = @(x,f,g,xinterp) interp_qcmixed(x,f,g,xinterp,0.8);
plotcurve(Msigma(1:end-1,:),Mlambda(1:end-1,:),MPareto, interpfun, ...
          'interp_qcmixed',...
          'one-norm of solution','two-norm of residual',...
          name,printflag,legendflags(4),4);

% Approximation using cubic interpolation, including BP solution
plotcurve(Msigma,Mlambda,MPareto, @interp_qcl, ...
          'interp_qcl, all tau samples',...
          'one-norm of solution','two-norm of residual',...
          name,printflag,legendflags(5),5);

end
  

function plotcurve(Msigma,Mlambda,MPareto,interpfun, ...
                   header,xtitle,ytitle,name,printflag,legendflag,figno)

% Legend keys
keys = {'Pareto curve', ...
        'Sigma-based approximation', ...
        'Lambda-based approximation', ...
        'Sigma-based samples', ...
        'Lambda-based samples'};

% Points for interpolation
x = MPareto(1,1);
for i=2:size(MPareto,1);
  pts = linspace(MPareto(i-1,1),MPareto(i,1),20);
  x   = [x, pts(2:end)];
end

% Plot the Pareto curve
figure(figno); clf; gca; hold on;
h = plot(MPareto(:,1), MPareto(:,2), 'k-');

% Interpolate the curve is function is given
if isempty(interpfun)
   keys = keys([1,4,5]);
else
  y1 = interpfun(Msigma(:,1), Msigma(:,2), Msigma(:,3), x);
  y2 = interpfun(Mlambda(:,1),Mlambda(:,2),Mlambda(:,3),x);
  
  hp = plot(x,y1,'b--', x,y2,'r-.');
  h  = [h; hp];
  
end;

% Plot the sample points
hp = plot(Msigma(:,1),  Msigma(:,2),    'bo', ...
          Mlambda(:,1), Mlambda(:,2), 'r*');
h = [h; hp];

% Set figure properties
set(gcf,'Name',header);
set(gca,'FontSize',18);
ylim([0,1.1*max(MPareto(:,2))]);
if legendflag, legend(keys); end;
xlabel(xtitle);
ylabel(ytitle);

% Export plot to file
if printflag
   set(gca,'FontSize',18);
   set(h,'MarkerSize',12,'LineWidth',2);
   fn  = sprintf('figures/ParetoExtrap_%s_%s.pdf',name,char('a' + figno - 1));
   exportPDF(fn);
end

end
