function QPLSplot(problem, varargin)
%QPLSPLOT  Generates QPLS comparison plots.
%
%   QPLSPLOT(FLAGS,OPTIONS), all FLAGS and OPTIONS are optional
%   and have default values. All FLAGS are false unless set. Valid
%   FLAGS are
%
%   'export'     - Output results to file
%   'nogenerate' - Suppress generating data files when missing
%   
%   QPLSPLOT itself does not process any OPTIONS, but they are
%   passed on to QPLSGEN and QPLSBATCH to generate the plot
%   data. The most useful option is 'parallel' which takes
%   advantage of the parfor command, if present, when generating
%   data.
%
%   Examples:
%   QPLSplot(5,'nogenerate'); % Show QPLS results for problem 5
%                             % without generating missing data.
%
%Paper: Probing the Pareto Frontier for Basis Pursuit Solutions

%   Copyright 2008, Ewout van den Berg and Michael P. Friedlander
%   $Id$

% ---------------------------------------------------------------


% Parse parameters
[opts,varg] = parseOptions(varargin, {'nogenerate','export'},{});

% Get problem identifier
if nargin < 1, error('QPLSplot requires at least a problem ID'); end;
name = generateProblem(problem,'getname');

% Get problem list
[probList,solverList,fileList] = QPLSgen(problem);

% Get Pareto curve
[probList,filename] = PARETOgen(problem,varg{:});
if opts.nogenerate && ~exist(filename,'file')
  pareto = [];  
else
  pareto = PARETObatch(problem,varg{:});
end
  
% Set solvers
if opts.nogenerate
  sl = {};
  for i=1:length(solverList)
    if exist(fileList{i},'file')
      sl{end+1} = solverList{i};
    end
  end
  solverList = sl;  
end

% Get data
qplsstats = QPLSbatch(problem,'solvers',solverList,varg{:});

% Set plot settings
solvers = {'gpsr', 'spgl1','l1ls', 'l1magic','pdco','fpc'};
key     = {'GPSR', 'SPGL1','L1LS', 'L1Magic','PDCO','FPC'};
style   = {'rx'  , 'bo'   , 'r.' , 'bx',     'r^'  , 'g.' };
fs      = 18; % Fontsize;
ms      = 14;  % Marker size;


% ----------------------------------------------------------------------
% Output plots
% ----------------------------------------------------------------------

% Gather all available data
keys    = {};
markers = {};
stats   = {};

for i=1:length(qplsstats)
   if isempty(qplsstats{i}) || ...
      isempty(qplsstats{i}.xNorm)
     continue;
   end;

   % Check if solver is recognized
   [member,idx] = ismember(qplsstats{i}.solver,solvers);
   if ~member, continue; end;
   
   keys{end+1}    = key{idx};
   markers{end+1} = style{idx};
   stats{end+1}   = qplsstats{i};
end
  
% Sort results by solver name
[keys,idx] = sort(keys);
markers    = markers(idx);
stats      = stats(idx);

% ----------------------------------------------------------------------
% Plot 1
% ----------------------------------------------------------------------
figure; clf; gca; hold on; legendkeys = {};

% Figure 1 - Output Pareto curve
if ~isempty(pareto)
   plot(pareto.tau, abs(pareto.rNorm), 'k-');
   legendKeys{1} = 'Pareto curve';
end

% Figure 1 - Output all curves
for i=1:length(solverList)
   idx = find(~cellfun('isempty',stats{i}.err));
   stats{i}.xNorm(idx) = NaN;
   stats{i}.rNorm(idx) = NaN;
   h = plot(stats{i}.xNorm,stats{i}.rNorm,markers{i});
   set(h,'MarkerSize',ms);
end
hold off;
  
% Figure 1 - Output the legend
legendkeys = {legendKeys{:},keys{:}};
h = legend(legendkeys); set(h,'Fontsize',fs,'Location','SouthWest');

% Figure 1 - Set axis properties
xlim([0,1.05*max(pareto.tau)]); ylim([0,max(pareto.rNorm)*1.05]);
set(gca,'Fontsize',fs,'YScale','log');
xlabel('one-norm of solution');
ylabel('two-norm of residual');
box off;

% Figure 1 - Export to File
if opts.export
   fn = sprintf('./figures/QPLSplot_%s_a.pdf',name);
   exportPDF(fn);
   close(gcf);
end
  
      
% ----------------------------------------------------------------------
% Plot 1
% ----------------------------------------------------------------------
figure; clf; gca; hold on;

% Figure 2 - Output all curves
for i=1:length(solverList)
   h = plot(stats{i}.tau,stats{i}.matvec,markers{i});
   set(h,'MarkerSize',ms);
end
hold off;

% Figure 2 - Set axis properties
xlim([0,1.05*max(pareto.tau)]);
set(gca,'Fontsize',fs,'YScale','log');
xlabel('one-norm of solution');
ylabel('number matrix-vector products');
box off;

% Figure 2 - Export to File
if opts.export
   fn = sprintf('./figures/QPLSplot_%s_b.pdf',name);
   exportPDF(fn);
   close(gcf);
end
