function figureParetoNewton(exportflag)
%FIGUREPARETONEWTON  Illustrates Newton on Pareto curve
%
%   FIGUREPARETONEWTON(EXPORTFLAG) plots an illustration of
%   Newton root finding on the Pareto curve for Sparco problem
%   11. When the EXPORTFLAG is set the results are output to PDF
%   file.
%
%Paper: Probing the Pareto Frontier for Basis Pursuit Solutions

%   Copyright 2008, Ewout van den Berg and Michael P. Friedlander
%   $Id$

% ---------------------------------------------------------------

% Get parameters
if nargin < 1, exportflag = 0; end;

% Generate test problem
P    = generateProblem(11);
P.x0 = P.x0 / 4;
P.b  = P.b / 4;

% Compute the Pareto curve
tau   = linspace(0,norm(P.x0,1),21);
xNorm = zeros(size(tau));
rNorm = zeros(size(tau));
rGap  = zeros(size(tau));
gNorm = zeros(size(tau));

for i=1:length(tau)
  opts.verbosity = 0;
  [x,r,g,info] = spgl1(P.A,P.b,tau(i),[],[],opts);
  xNorm(i) = norm(x,1);
  rNorm(i) = norm(r,2);
  rGap(i)  = info.rGap;
  gNorm(i) = info.gNorm;
end;

% Set desired level of sigma
sigma = 3;

% Plot curve
figure(1); clf; gca; hold on;
h1 = plot(xNorm,rNorm,'b-');
h2 = plot(linspace(0,7,101),ones(1,101)*sigma,'k:');
h = [h1;h2];

% Take two Newton steps
tau = 0;
for i=1:2
  opts.verbatim = 0;
  [x,r,g,info] = spgl1(P.A,P.b,tau,[],[],opts);

  x = norm(x,1);
  f = norm(r,2);
  g = -info.gNorm / f;
  d = (sigma - f) / g;
  
  % Plot line
  h3 = plot([x,x+d],[f,sigma],'r--', ...
            [x,x+d],[f,sigma],'r.');
  if (i ~= 1)
    h4 = plot([x,x],[sigma,f],'r:');
    h  = [h; h3; h4];
  else
    h  = [h; h3];
  end
  
  % Set new tau
  tau = tau + d;
end

% Set plot properties
set(gca,'FontSize',18);
set(h,'LineWidth',2,'MarkerSize',20);
xlabel('one-norm of solution');
ylabel('two-norm of residual');

if exportflag
  exportPDF('./figures/figureParetoNewton.pdf');
end
