function stats = BPDNbatch(varargin)
%BPDNbatch  Run BPDN problems and store data
%
%   STATS = BPDNBATCH(FLAGS,OPTIONS), all FLAGS and OPTIONS are
%   optional and have default values. All FLAGS are false unless
%   set. Valid FLAGS are
%
%   'parallel'   - Take advantage of the parfor statement to speed
%                  up computations where possible
%   
%   OPTIONS consist of a keyword followed by a value:
%
%   'problems' - [BPDNgen()] gives a list of all problems to run
%
%   The output variable STATS is a cell array of structures.
%   BPDNBATCH always checks if data for the problem is already
%   available on disk to avoid unnecessarily rerunning experiments.
%
%   Examples:
%   s = BPDNbatch();  % Run all
%
%Paper: Probing the Pareto Frontier for Basis Pursuit Solutions

%   Copyright 2008, Ewout van den Berg and Michael P. Friedlander
%   $Id$

% ---------------------------------------------------------------

% Parse parameters
[opts,varg] = parseOptions(varargin,{'parallel'},{'problems'});
probList    = getOption(opts,'problems',BPDNgen(varg{:}));

% Run problems
n = length(probList);
if opts.parallel
  stats = ParallelBatch(probList,@runprob);
else
  stats = {};
  for i=1:n
      p = probList{i};
      stats{i} = runprob(p);
  end
end

end % end BPDNbatch


function stats = runprob(p)
    solver = p{1};
    relres = p{2};
    pID    = p{3};
    fn     = p{4};

    if exist(fn,'file')
        s = load(fn,'stats');
        stats = s.stats;
    else
        s = generateProblem(pID);
        bNorm = norm(s.b);
        sigma = relres * bNorm;
        stats = BPDNrun(s.A,s.b,sigma,solver);
        stats.name = s.info.name;
        save(fn,'p','stats');
    end
end
