function stats = BPDNrun(A,b,sigma,solver)
%BPDNRUN  Solves a BPDN problem
%
%   STATS = BPDNRUN(A,B,SIGMA,SOLVER) solves the BPDN problem
%   given by matrix A, vector B and tolerated residual norm
%   SIGMA using solver SOLVER. Valid solvers are 'spgl1',
%   'sparselab', and 'l1magic'.
%
%Paper: Probing the Pareto Frontier for Basis Pursuit Solutions

%   Copyright 2008, Ewout van den Berg and Michael P. Friedlander
%   $Id$


% Parameters
timeAllowed = 3600; % One hour maximum allowed runtime.

% Initialize
infoA = A([],0);
[m, n] = infoA{1:2};
err = [];           % reset error flag
out = 'FAILED';     % default log
x   = zeros(n,1);   % default sol'n
info = [];          % solver info (if any)

% Start the clocks!
timeStamp = now;    % Use datestr to make human readable.
tic;

switch lower(solver)
    % ---------------------------------------------------------------------
    case 'spgl1'
        resetMatvec;
        tau  = 0;  % Initial value of tau.
        x0   = zeros(n,1);
        opts = spgSetParms('verbosity',2);
        tic;
        try
            cmd = 'spgl1(@opMatvec, b, [], sigma, x0, opts);';
            [out,x,r,g,info] = evalc(cmd);
        catch
          err = lasterror;
        end
  
    % ---------------------------------------------------------------------
    case 'sparselab'
        resetMatvec;
        lamStop = 0;     % Stop when Lagrange mult is zero
        resStop = sigma; % Stop when residual is less than sigma
        maxItns = 1e4;   % Max no. of iterations
        solFreq = 0;     % Only return the final solution
        verbose = 1;     % No output.
        tic;
        try
            cmd = ['SolveLasso(@opSparselab, b, n, ''lasso'',', ...
                   '            maxItns, lamStop, resStop, ', ...
                   '            solFreq, verbose );'];
            [out,x] = evalc(cmd);
        catch
          err = lasterror;
        end            
        
    % ---------------------------------------------------------------------
    case 'l1magic'
        resetMatvec;
        x0 = opMatvecAT(b);
        tic;
        try
            cmd = ['l1qc_logbarrier(x0, @opMatvecA, ', ...
                   '                @opMatvecAT, b, sigma);'];
            [out,x] = evalc(cmd);
        catch
          err = lasterror;
        end
  
    % ---------------------------------------------------------------------
    otherwise
        error('solver %s not recognized',solver)
        
end % switch

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compute solution stats.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

runTime = toc;
r = b - A(x,1);
xNorm1 = norm(x,1);
rNorm2 = norm(r,2);
gNormI = norm(A(r,2),inf);

% Duality gap for LASSO problem. (Pretend x,r candidate sol of Lasso.)
pVal = r'*r/2;
dVal = b'*r - r'*r/2 - xNorm1*gNormI;
rgapLasso = (pVal - dVal) / max([1,pVal,dVal]);

% Duality gap for BPDN.
pVal = xNorm1;
if gNormI < 1e-6
   dVal = 0;
else
   dVal = (b'*r - .5*sigma^2 - .5*r'*r)/gNormI;
end
rgapBPDN = (pVal - dVal) / max([1,pVal,dVal]);

% Save run stats.
stats.x = x;
stats.solver    = solver;
stats.timeStamp = datestr(timeStamp);
stats.log       = out;
stats.runTime   = runTime;
stats.matvec    = [ nAx, nATx ];
stats.gapLasso  = rgapLasso;
stats.gapBPDN   = rgapBPDN;
stats.xNorm     = xNorm1;
stats.rNorm     = rNorm2;
stats.bNorm     = norm(b);
stats.sigma     = sigma;
stats.m         = m;
stats.n         = n;
stats.err       = err;
stats.info      = info;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% NESTED FUNCTIONS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function resetMatvec()
   nAx  = 0;
   nATx = 0;
   evalin('base','matvec = [0,0];');
   evalin('base','matvec = [0,0];');
end

function y = Aprod(x,mode)
% Master Mat-vec routine.  Should be called by all operators.
  timeElapsed = toc;
  if timeElapsed >= timeAllowed
    error('BPDNrun:timeOut','Out of time!')
  end
  y = A(x,mode);
end

function y = opMatvecA(x)
   nAx = nAx + 1;
   y   = Aprod(x,1);
end

function y = opMatvecAT(x)
   nATx = nATx + 1;
   y    = Aprod(x,2);
end
   
function y = opMatvec(x,mode)
   if    mode == 1,  nAx  = nAx  + 1;
   else              nATx = nATx + 1;
   end
   y = Aprod(x,mode);
end
   
function y = opSparselab(mode,m,nIdx,x,idx,n)
  if mode == 1
    nAx = nAx + 1;
    z = zeros(n,1);
    z(idx) = x;
    y = Aprod(z,1);
  else
    nATx = nATx + 1;
    z = Aprod(x,2);
    y = z(idx);
  end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
end % BPDNrun
