function stats = BPrun(A,b,solver)
%BPRUN  Solves a BP problem
%
%   STATS = BPRUN(A,B,SOLVER) solves the BP problem given by matrix
%   A, and vector B using solver SOLVER. Valid solvers are 'spgl1',
%   'spgl1-2', 'sparselab', 'pdco', and 'l1magic'.
%
%Paper: Probing the Pareto Frontier for Basis Pursuit Solutions

%   Copyright 2008, Ewout van den Berg and Michael P. Friedlander
%   $Id$


% Parameters
timeAllowed = 3600; % Two hours maximum allowed runtime.

% Initialize
infoA = A([],0);
[m, n] = infoA{1:2};
err = [];  % reset error flag
out = 'FAILED';     % default log
x   = zeros(n,1);   % default sol'n
info = [];          % default solver info (if any)

% Start the clocks!
timeStamp = now;      % Use datestr to make human readable.
tic;

switch lower(solver)
    % ---------------------------------------------------------------------
    case 'spgl1'
        resetMatvec;
        tau  = 0;  % Initial value of tau.
        x0   = zeros(n,1);
        opts = spgSetParms('verbosity',2);
        tic;
        try
            cmd = 'spgl1(@opMatvec, b, 0, 0, x0, opts);';
            [out,x,r,g,info] = evalc(cmd);
        catch
          err = lasterror;
        end
        runTime = toc;
    
    % ---------------------------------------------------------------------
    case 'spgl1-2'
        resetMatvec;
        tau  = 0;  % Initial value of tau.
        x0   = zeros(n,1);
        opts = spgSetParms('verbosity',2,'subspace',1);  % Turn on subspace
        tic;
        try
            cmd = 'spgl1(@opMatvec, b, 0, 0, x0, opts);';
            [out,x,r,g,info] = evalc(cmd);
        catch
          err = lasterror;
        end
        runTime = toc;
  
    % ---------------------------------------------------------------------
    case 'sparselab'
        resetMatvec;
        lamStop = 0;     % Stop when Lagrange mult is zero
        resStop = 0;     % Stop when residual is less than sigma
        maxItns = 1e4;   % Max no. of iterations
        solFreq = 0;     % Only return the final solution
        verbose = 1;     % No output.
        tic;
        try
            cmd = ['SolveLasso(@opSparselab, b, n, ''lasso'',', ...
                   '            maxItns, lamStop, resStop, ', ...
                   '            solFreq, verbose );'];
            [out,x] = evalc(cmd);
        catch
          err = lasterror;
        end
        runTime = toc;
        
    % ---------------------------------------------------------------------
    case 'pdco'
        resetMatvec;
        maxItns = 1e4;   % Max. no. of iterations
        tic;
        try
            cmd = 'SolveBP(@opSparselab, b, n, maxItns);';
            [out,x] = evalc(cmd);
        catch
          err = lasterror;
        end            
        runTime = toc;
     
    % ---------------------------------------------------------------------
    case 'l1magic'
        resetMatvec;
        x0 = opMatvecAT(b);
        tic;
        try
            cmd = 'l1eq_pd(x0, @opMatvecA, @opMatvecAT, b);';
            [out,x] = evalc(cmd);
        catch
          err = lasterror;
        end
        runTime = toc;
  
    % ---------------------------------------------------------------------
    otherwise
        error('solver %s not recognized',solver)
end % switch

r = b - A(x,1);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compute solution stats.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

xNorm1 = norm(x,1);
rNorm2 = norm(r,2);
gNormI = norm(A(r,2),inf);

% Save run stats.
stats.x = x;
stats.solver    = solver;
stats.timeStamp = datestr(timeStamp);
stats.log       = out;
stats.runTime   = runTime;
stats.matvec    = [ nAx, nATx ];
stats.xNorm     = xNorm1;
stats.rNorm     = rNorm2;
stats.bNorm     = norm(b);
stats.m         = m;
stats.n         = n;
stats.err       = err;
stats.info      = info;

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% NESTED FUNCTIONS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function resetMatvec()
   nAx  = 0;
   nATx = 0;
   evalin('base','matvec = [0,0];');
   evalin('base','matvec = [0,0];');
end

function y = Aprod(x,mode)
% Master Mat-vec routine.  Should be called by all operators.
  timeElapsed = toc;
  if timeElapsed >= timeAllowed
    error('BPDNrun:timeOut','Out of time!')
  end
  y = A(x,mode);
end

function y = opMatvecA(x)
   nAx = nAx + 1;
   y   = Aprod(x,1);
end

function y = opMatvecAT(x)
   nATx = nATx + 1;
   y    = Aprod(x,2);
end
   
function y = opMatvec(x,mode)
   if    mode == 1,  nAx  = nAx  + 1;
   else              nATx = nATx + 1;
   end
   y = Aprod(x,mode);
end
   
function y = opSparselab(mode,m,nIdx,x,idx,n)
  if mode == 1
    nAx = nAx + 1;
    z = zeros(n,1);
    z(idx) = x;
    y = Aprod(z,1);
  else
    nATx = nATx + 1;
    z = Aprod(x,2);
    y = z(idx);
  end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
end % BPrun
