function stats = PARETObatch(problem,varargin)
%PARETObatch  Generate accurate samples along the Pareto curve
%
%   STATS = PARETOBATCH(PROBLEM,FLAGS,OPTIONS), generates the
%   Pareto curve for problem PROBLEM. All FLAGS are optional and
%   have default values. All FLAGS are false unless set. Valid
%   FLAGS are
%
%   'parallel'   - Take advantage of the parfor statement to speed
%                  up computations where possible
%   
%   The output variable STATS is a cell array of structures.
%   PARETObatch always checks if data for the problem is already
%   available on disk to avoid unnecessarily rerunning
%   experiments.
%
%   Examples:
%   s = PARETObatch(3);  % Sample curve for Sparco problem 3
%   s = PARETObatch('cosspike'); % Same
%
%Paper: Probing the Pareto Frontier for Basis Pursuit Solutions

%   Copyright 2008, Ewout van den Berg and Michael P. Friedlander
%   $Id$

% ---------------------------------------------------------------

% Check parameters
if nargin < 1
  error('PARETObatch requires at least a problem ID');
end;

% Parse parameters
[opts,varg] = parseOptions(varargin,{'parallel'},{});
[probList,filename] = PARETOgen(problem, varg{:});

% Check if given problem name or ID is valid
if isempty(probList), stats = []; return; end

% Check if results are available
if exist(filename,'file')
   s = load(filename,'pstats');
   stats = s.pstats;
   return;
end


% --------------------------------------
% Ensure Basis Pursuit norm is available
% --------------------------------------

% Generate TAUBP problem list and get solutions
name       = probList{1}.problem;
taubpprob = TAUBPgen(varg{:},'problems',name);
taubpstat = TAUBPbatch(varg{:},'problems',taubpprob);

% Compute the tau value for each problem
s     = taubpstat{1};
taubp = s.tauHist(end);
for i=1:length(probList)
   entry = probList{i};
   entry.tau = entry.fraction * taubp;
   probList{i} = entry;
end


% ------------------------------------
% Run problems
% ------------------------------------
n = length(probList);
if opts.parallel
  stats = ParallelBatch(probList,@runprob);
else
  stats = {};
  for i=1:n
      p = probList{i};
      stats{i} = runprob(p);
  end
end


% ---------------------------
% Collect data in single file
% ---------------------------

% Initialize stats structure
n = length(probList);
pstats.tau     = zeros(n,1);
pstats.xNorm   = zeros(n,1);
pstats.rNorm   = zeros(n,1);
pstats.rGap    = zeros(n,1);
pstats.stat    = zeros(n,1);
pstats.lambda  = zeros(n,1);
pstats.problem = name;
    
% Get information
for i=1:n
  pstats.tau(i)    = stats{i}.tau;
  pstats.xNorm(i)  = stats{i}.xNorm;
  pstats.rNorm(i)  = stats{i}.rNorm;
  pstats.rGap(i)   = stats{i}.rGap;
  pstats.stat(i)   = stats{i}.stat;
  pstats.lambda(i) = stats{i}.lambda;
end
    
% Write file
save(filename,'pstats');
    
% Delete intermediate files
for i=1:n
   fn = probList{i}.filename;
   cmd = sprintf('delete %s;',fn);
   eval(cmd);
end

% Return data
stats = pstats;

end % end PARETObatch



function stats = runprob(p)
    if exist(p.filename,'file')
        s = load(p.filename,'stats');
        stats = s.stats;
    else
        s = generateProblem(p.problem);
        stats = PARETOrun(s.A,s.b,p.tau);
        save(p.filename,'p','stats');
    end
end
