function [probList,filename] = PARETOgen(problem,varargin)
%PARETOGEN  Generates list of Pareto problems to run
%
%   [PROBLIST,FILENAME] = PARETOGEN(PROBLEM,OPTIONS), generates a
%   list of sub-problems for PROBLEM. All OPTIONS are optional and
%   have default values. OPTIONS consist of a keyword followed by a
%   value:
%
%   'path'     - ['data'] directory to store results in
%   'prefix'   - ['Pareto'] prefix of filename
%
%   PROBLIST is a cell array of structs. The entries of each struct
%   are 'problem', 'fraction', 'filename' for problem name,
%   fraction of tau(BP) to use, and filename for intermediate
%   data. FILENAME contains the name of the file in which the data
%   for all points on the Pareto curve are stored.
%
%   Examples:
%   plist = PARETOgen();              % Generate default list
%   plist = PARETOgen('path','data'); % Store results in 'data' dir
%
%Paper: Probing the Pareto Frontier for Basis Pursuit Solutions

%   Copyright 2008, Ewout van den Berg and Michael P. Friedlander
%   $Id$

% ---------------------------------------------------------------

% Check parameters
if nargin < 1
  error('PARETOgen requires at least a problem ID');
end;

% Parse parameters, set options
[opts,varg] = parseOptions(varargin,{},{'path','prefix'});
path        = getOption(opts,'path','data');
prefix      = getOption(opts,'prefix','Pareto');
fraction    = linspace(0,0.98,50);

% Convert problem number to name
if ~ischar(problem) && (length(problem) > 1)
     error('PARETOgen handles only one problem at a time');
end
if iscell(problem), problem = problem{1}; end;
problem = generateProblem(problem,'getname');


% Generate problem list
probList = {}; filename = []; idx = 1;
if isempty(problem), return; end;

for j = 1:length(fraction)
   fn = sprintf('%s%s%s_%s_%03d.mat',path,filesep,prefix,problem,j);
   entry          = struct();
   entry.problem  = problem;
   entry.fraction = fraction(j);
   entry.filename = fn;
   probList{idx} = entry;
   idx = idx + 1;
end

filename = sprintf('%s%s%s_%s.mat',path,filesep,prefix,problem);
