function stats = QPLSbatch(problem,varargin)
%QPLSbatch  Run QPLS problems and store data
%
%   STATS = QPLSBATCH(FLAGS,OPTIONS), all FLAGS and OPTIONS are
%   optional and have default values. All FLAGS are false unless
%   set. Valid FLAGS are
%
%   'parallel'   - Take advantage of the parfor statement to speed
%                  up computations where possible
%   
%   OPTIONS consist of a keyword followed by a value:
%
%   'solvers'    - [ all ] gives a list of solvers to use
%
%   The output variable STATS is a cell array of structures, one
%   for each solver. QPLSBATCH always checks if data for the
%   problem is already available on disk to avoid unnecessarily
%   rerunning experiments.
%
%   Examples:
%   s = QPLSbatch();  % Run all
%
%Paper: Probing the Pareto Frontier for Basis Pursuit Solutions

%   Copyright 2008, Ewout van den Berg and Michael P. Friedlander
%   $Id$

% ---------------------------------------------------------------

% Check parameters
if nargin < 1
  error('QPLSbatch requires at least a problem ID');
end;

% Parse parameters (solvers parameter dealt with by QPLSgen)
[opts,varg] = parseOptions(varargin,{'parallel'},{});
[probList,solvers,filenames] = QPLSgen(problem, varg{:});

% Check if given problem name or ID is valid
if isempty(probList), stats = {}; return; end

% Extract all problems for which the solution exists
pl = {}; idx = 1;
for i=1:length(filenames)
   if exist(filenames{i},'file'), continue; end;
   solver = solvers{i};
   for j=1:length(probList)
      if strcmp(probList{j}.solver,solver)
         pl{idx} = probList{j};
         idx = idx + 1;
      end
   end
end
probList = pl;


% ---------------------
% Generate missing data
% ---------------------
stats = cell(length(solvers),1);

if ~isempty(probList)
   % Get Pareto curve data
   parg = {}; if opts.parallel, parg = {'parallel'}; end;
   pareto = PARETObatch(problem,parg{:});
   
   % Add Pareto data to problem List
   for i=1:length(probList)
     entry = probList{i};
     index = entry.index;
     entry.tau = pareto.tau(index);
     entry.lambda = pareto.lambda(index);
     probList{i} = entry;
   end
   
   % Run problems
   n = length(probList);
   if opts.parallel
      pstats = ParallelBatch(probList,@runprob);
   else
      pstats = {};
      for i=1:n
         p = probList{i};
         pstats{i} = runprob(p);
      end
   end
end

% Collect all information
stats = {};
for i=1:length(solvers)
  solver = solvers{i};
  if ~exist(filenames{i},'file')
     % Extract all required information from pstats
     qplsstats = struct();
     qplsstats.solver   = solver;
     qplsstats.problem  = probList{1}.problem;
     qplsstats.matvec   = [];
     qplsstats.runtime  = [];
     qplsstats.xNorm    = [];
     qplsstats.rNorm    = [];
     qplsstats.lambda   = [];
     qplsstats.tau      = [];
     qplsstats.err      = {};
     qplsstats.info     = {};
     
     for j=1:length(probList)
       if ~strcmp(probList{j}.solver,solver), continue; end;
       
       idx = probList{j}.index;
       qplsstats.matvec(idx) = pstats{j}.matvec;
       qplsstats.runTime(idx)= pstats{j}.runTime;
       qplsstats.xNorm(idx)  = pstats{j}.xNorm;
       qplsstats.rNorm(idx)  = pstats{j}.rNorm;
       qplsstats.info{idx}   = pstats{j}.info;
       qplsstats.err{idx}    = pstats{j}.err;
       qplsstats.lambda(idx) = probList{j}.lambda;
       qplsstats.tau(idx)    = probList{j}.tau;
     end
     
     % Store data to file
     save(filenames{i},'qplsstats');
     
     % Delete intermediate files
     for j=1:length(probList)
        if ~strcmp(probList{j}.solver,solver), continue; end;
        if exist(probList{j}.filename,'file')
           cmd = sprintf('delete %s', probList{j}.filename);
           eval(cmd);
        end        
     end
     
     % Set result
     stats{i} = qplsstats;
  else
     % Load data directly from file
     s = load(filenames{i},'qplsstats');
     stats{i} = s.qplsstats;
  end
  
end

end % end QPLSbatch


function stats = runprob(p)
    if exist(p.filename,'file')
        fprintf('Loading %s\n', p.filename);
        s = load(p.filename,'stats');
        stats = s.stats;
    else
        fprintf('Generating %s\n', p.filename);
        s = generateProblem(p.problem);
        qstats = QPLSrun(s.A,s.b,p.lambda,p.tau,p.solver);
        
        % Copy only relevant entries
        stats.runTime = qstats.runTime;
        stats.matvec  = qstats.matvec;
        stats.xNorm   = qstats.xNorm;
        stats.rNorm   = qstats.rNorm;
        stats.stat    = qstats.stat;
        stats.info    = qstats.info;
        stats.err     = qstats.err;
        
        save(p.filename,'p','stats');
    end
end
