function stats = QPLSrun(A,b,lambda,tau,solver)
%QPLSRUN  Solves a QPlambda or LStau problem
%
%   STATS = QPLSRUN(A,B,LAMBDA,TAU,SOLVER) solves the QPlambda or
%   LStau problem given by matrix A, vector B and parameters LAMBDA
%   or TAU, using solver SOLVER. Whether QPlambda or LStau is
%   solved depends on the choice of SOLVER. Valid solvers are
%   'gpsr','spgl1','homotopy','pdco','l1ls', and 'fpc'.
%
%Paper: Probing the Pareto Frontier for Basis Pursuit Solutions

%   Copyright 2008, Ewout van den Berg and Michael P. Friedlander
%   $Id$

% Parameters
timeAllowed = 17400; % Four hours maximum allowed runtime.

% Initialize
infoA  = A([],0);
[m, n] = infoA{1:2};
err    = [];           % reset error flag
out    = 'FAILED';     % default log
x      = zeros(n,1);   % default sol'n
info   = [];           % solver info (if any)
nAx    = 0;
nATx   = 0;
resetMatvec;

% Solve the problem
timeStamp = now;      % Use datestr to make human readable.
tic;

switch lower(solver)
    % ---------------------------------------------------------------------
    case 'gpsr'
        try
           args = {b,@opMatvecA,lambda,'AT',@opMatvecAT,'Verbose',0};
           [out,x,x_debias,obj] = evalc('GPSR_BB(args{:});');
        catch
           err = lasterror;
        end

    % ---------------------------------------------------------------------
    case 'spgl1'
        opts = spgSetParms('verbosity',1,'subspace',0);
        try
            cmd = 'spgl1(@opMatvec, b, tau, [], x, opts);';
            [out,x,r,g,info] = evalc(cmd);
        catch
            err = lasterror;
        end

    % ---------------------------------------------------------------------
    case 'homotopy'
        lamStop = lambda;% Stop when Lagrange mult is zero
        resStop = 0;     % Stop when residual is less than sigma
        maxItns = 1e4;   % Max no. of iterations
        solFreq = 0;     % Only return the final solution
        verbose = 1;     % Verbose.
        tic;
        try
            cmd = ['SolveLasso(@opSparselab, b, n, ''lasso'',', ...
                   '            maxItns, lamStop, resStop, ', ...
                   '            solFreq, verbose );'];
            [out,x] = evalc(cmd);
        catch 
          err = lasterror;
        end

    % ---------------------------------------------------------------------
    case 'pdco'
        maxItns = 1e4;   % Max. no. of iterations
        try
          args = {@opSparselab,b,n,maxItns,lambda};
          [out,x] = evalc('SolveBP(args{:});');
        catch
           err = lasterror;
        end

    % ---------------------------------------------------------------------
    case 'l1ls'
        cA     = classOp(A,'matvec');
        cAT    = cA';
        optTol = 1e-3; % Default value
        try
          args = {cA,cAT,m,n,b,2*lambda,optTol,1};
          [out,x,status,history] = evalc('l1_ls(args{:});');
        catch
           err = lasterror;
        end

    % ---------------------------------------------------------------------
    case 'fpc'
        M = [];
        tic;
        try
            cmd = 'fpc(n,@opFPC,b,1/lambda,M,[]);';
            [out,result] = evalc(cmd);
            x = result.x;
        catch
          err = lasterror;
        end

    % ---------------------------------------------------------------------
    otherwise
        error('solver %s not recognized',solver)

end % switch

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compute solution stats.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

runTime = toc;
matvec  = getMatvec();
r       =  b - A(x,1);
xNorm1  = norm(x,1);
rNorm2  = norm(r,2);
gNormI  = norm(A(r,2),inf);

% Set results
stats = struct();
stats.solver    = solver;
stats.timeStamp = datestr(timeStamp);
stats.log       = out;

stats.runTime = runTime;
stats.matvec  = sum(matvec);
stats.xNorm   = xNorm1;
stats.rNorm   = rNorm2;
stats.bNorm   = norm(b);
stats.m       = m;
stats.n       = n;
stats.info    = info;
stats.err     = err;
stats.stat    = isempty(err);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% NESTED FUNCTIONS
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function resetMatvec()
   nAx  = 0;
   nATx = 0;
   evalin('base','matvec = [0,0];');
   evalin('base','matvec = [0,0];');
end

function n = getMatvec()
   matvec = evalin('base','matvec');
   n = matvec + [nAx, nATx];
end

function y = Aprod(x,mode)
% Master Mat-vec routine.  Should be called by all operators.
  timeElapsed = toc;
  if timeElapsed >= timeAllowed
    error('BPDNrun:timeOut','Out of time!')
  end
  y = A(x,mode);
end

function y = opMatvecA(x)
   nAx = nAx + 1;
   y   = Aprod(x,1);
end

function y = opMatvecAT(x)
   nATx = nATx + 1;
   y    = Aprod(x,2);
end
   
function y = opMatvec(x,mode)
   if    mode == 1,  nAx  = nAx  + 1;
   else              nATx = nATx + 1;
   end
   y = Aprod(x,mode);
end

function y = opSparselab(mode,m,nIdx,x,idx,n)
  if mode == 1
    nAx = nAx + 1;
    z = zeros(n,1);
    z(idx) = x;
    y = Aprod(z,1);
  else
    nATx = nATx + 1;
    z = Aprod(x,2);
    y = z(idx);
  end
end

function y = opFPC(trans,m,n,x,idx,varargin)
  if ~trans
    nAx = nAx + 1;
    if isempty(idx)
        z = x;
    else
        z = zeros(n,1);
        z(idx) = x;
    end
    y = Aprod(z,1);
  else
    nATx = nATx + 1;
    z = Aprod(x,2);
    if isempty(idx)
        y = z;
    else
        y = z(idx);
    end
  end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
end % QPLSrun
