function stats = TAUBPrun(A,b)
%TAUBPRUN  Solves a TAUBP problem (get Basis Pursuit norm)
%
%   STATS = TAUBPRUN(A,B) accurately solves the BP problem given by
%   matrix A, and vector B to find tau(BP).
%
%Paper: Probing the Pareto Frontier for Basis Pursuit Solutions

%   Copyright 2008, Ewout van den Berg and Michael P. Friedlander
%   $Id$


% Check complexity of problem
info      = A([],0);
m         = info{1};
n         = info{2};
iscomplex = (info{3}(1) == 1) || ...
            (info{3}(3) == 1) || ...
            (~isreal(b));

% Initialize statistics
stats.tauHist   = [];
stats.rNormHist = [];
stats.gNormHist = [];
stats.rGapHist  = [];
stats.statHist  = [];
stats.iterHist  = [];

% Apply Newton root finding on the Pareto curve
opts = spgSetParms('iterations',  500,  ...
                   'optTol',      1e-5, ...
                   'bpTol',       1e-7, ...
                   'verbosity',   0,    ...
                   'subspaceMin', 0,    ...
                   'iscomplex',   iscomplex);

tau       = 0;
sigmaOld  = NaN;
xOld      = zeros(n,1);

for j=1:30 % Maximum number Newton iterations

   % -------------------------------------------
   % Solve LASSO(tau) subproblem
   % -------------------------------------------
   x = xOld;
   for i=1:40 % Maximum 40x500 = 20,000 iterations
      [x,r,g,info] = spgl1(A,b, tau,[],x, opts);
      if (info.stat ~= 1), break; end;
   end
 
   % -------------------------------------------
   % Store current values
   % -------------------------------------------
   stats.tauHist(end+1)   = tau;
   stats.iterHist(end+1)  = (i-1)*opts.iterations + info.iter;
   stats.rNormHist(end+1) = info.rNorm;
   stats.gNormHist(end+1) = info.gNorm;
   stats.rGapHist(end+1)  = info.rGap;
   stats.statHist(end+1)  = info.stat;

   % -------------------------------------------
   % Update tau
   % -------------------------------------------
   xOld = x;
   if (info.rNorm > 1e-5*norm(b,2))
      tau = tau + info.rNorm.^2 / info.gNorm;
   else
      status = 0; break;
   end;

   % Quit if sigma does not make much progress
   if ((sigmaOld - info.rNorm) / info.rNorm) < 1e-3
      status = 0; break;
   else
      sigmaOld = info.rNorm;
   end;
end

stats.x = x;

end % TAUBPrun
